@@ -29,8 +29,6 @@
 
 #define NORMAL_FONT_SCALE 0.9
 #define PAINTED_FONT_SCALE 0.6
-#define PAINTED_ALPHA 0.7
-#define NORMAL_ALPHA 1.0
 #define TAG_OFFSET 0.75
 #define TAG_RADIUS 0.25
 #define HINT_FLASHES 6
@@ -51,8 +49,7 @@ static void redo_cb (GSimpleAction *action, GVariant *parameter, gpointer user_d
 static void quit_cb (GSimpleAction *action, GVariant *parameter, gpointer user_data);
 static void help_cb (GSimpleAction *action, GVariant *parameter, gpointer user_data);
 static void about_cb (GSimpleAction *action, GVariant *parameter, gpointer user_data);
-static void board_size_activate_cb (GSimpleAction *action, GVariant *parameter, gpointer user_data);
-static void board_size_change_cb (GSimpleAction *action, GVariant *state, gpointer user_data);
+static void board_size_change_cb (GObject *object, GParamSpec *pspec, gpointer user_data);
 
 static GActionEntry app_entries[] = {
 	{ "new-game", new_game_cb, NULL, NULL, NULL },
@@ -62,7 +59,6 @@ static GActionEntry app_entries[] = {
 };
 
 static GActionEntry win_entries[] = {
-	{ "board-size", board_size_activate_cb, "s", "'5'", board_size_change_cb },
 	{ "hint", hint_cb, NULL, NULL, NULL },
 	{ "undo", undo_cb, NULL, NULL, NULL },
 	{ "redo", redo_cb, NULL, NULL, NULL },
@@ -74,8 +70,10 @@ hitori_create_interface (Hitori *hitori)
 	GError *error = NULL;
 	GtkBuilder *builder;
 	GtkStyleContext *style_context;
+	GtkCssProvider *css_provider;
 	const PangoFontDescription *font;
-	GMenuModel *app_menu, *win_menu;  /* owned */
+	GMenuModel *app_menu;  /* owned */
+	GAction *action;
 
 	builder = gtk_builder_new ();
 
@@ -106,29 +104,44 @@ hitori_create_interface (Hitori *hitori)
 	hitori->drawing_area = GTK_WIDGET (gtk_builder_get_object (builder, "hitori_drawing_area"));
 	hitori->timer_label = GTK_LABEL (gtk_builder_get_object (builder, "hitori_timer"));
 
-	/* Set up the menus (application and window). */
+	/* Set up the app menu */
 	app_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "app_menu"));
-	g_action_map_add_action_entries (G_ACTION_MAP (hitori), app_entries, G_N_ELEMENTS (app_entries), hitori);
 	gtk_application_set_app_menu (GTK_APPLICATION (hitori), app_menu);
 	g_object_unref (app_menu);
 
-	win_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "win_menu"));
+	g_object_unref (builder);
+
+	/* Set up actions */
+	g_action_map_add_action_entries (G_ACTION_MAP (hitori), app_entries, G_N_ELEMENTS (app_entries), hitori);
 	g_action_map_add_action_entries (G_ACTION_MAP (hitori->window), win_entries, G_N_ELEMENTS (win_entries), hitori);
-	gtk_application_set_menubar (GTK_APPLICATION (hitori), win_menu);
-	g_object_unref (win_menu);
+
+	action = g_settings_create_action (hitori->settings, "board-size");
+	g_action_map_add_action (G_ACTION_MAP (hitori), action);
+	g_signal_connect (G_OBJECT (action), "notify::state", (GCallback) board_size_change_cb, hitori);
+	g_object_unref (action);
 
 	hitori->undo_action = G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (hitori->window), "undo"));
 	hitori->redo_action = G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (hitori->window), "redo"));
 	hitori->hint_action = G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (hitori->window), "hint"));
 
-	g_object_unref (builder);
-
 	/* Set up font descriptions for the drawing area */
 	style_context = gtk_widget_get_style_context (hitori->drawing_area);
 	gtk_style_context_get (style_context, 0, GTK_STYLE_PROPERTY_FONT, &font, NULL);
 	hitori->normal_font_desc = pango_font_description_copy (font);
 	hitori->painted_font_desc = pango_font_description_copy (font);
 
+	/* Load CSS for the drawing area */
+	css_provider = gtk_css_provider_new ();
+	gtk_css_provider_load_from_path (css_provider, PACKAGE_DATA_DIR"/hitori/hitori.css", &error);
+	if (error) {
+		g_warning ("Unable to load CSS: %s", error->message);
+		g_error_free (error);
+	} else {
+		gtk_style_context_add_provider (style_context, GTK_STYLE_PROVIDER (css_provider),
+                                                GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+	}
+	g_object_unref (css_provider);
+
 	/* Reset the timer */
 	hitori_reset_timer (hitori);
 
@@ -136,14 +149,19 @@ hitori_create_interface (Hitori *hitori)
 	g_simple_action_set_enabled (hitori->undo_action, FALSE);
 	g_simple_action_set_enabled (hitori->redo_action, FALSE);
 
-	/* Set the initial board size. */
-	g_simple_action_set_state (G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (hitori->window), "board-size")), g_variant_new_string ("5"));
-
 	return hitori->window;
 }
 
 #define BORDER_LEFT 2.0
 
+static void
+lookup_color (GtkStyleContext *style_context, const gchar *name, GdkRGBA *result)
+{
+	if (!gtk_style_context_lookup_color (style_context, name, result)) {
+		g_warning ("Failed to look up color %s", name);
+	}
+}
+
 static void
 draw_cell (Hitori *hitori, GtkStyleContext *style_context, cairo_t *cr, gfloat cell_size, gdouble x_pos, gdouble y_pos,
            HitoriVector iter)
@@ -155,6 +173,7 @@ draw_cell (Hitori *hitori, GtkStyleContext *style_context, cairo_t *cr, gfloat c
 	gboolean painted = FALSE;
 	PangoFontDescription *font_desc;
 	GtkBorder border;
+	GdkRGBA colour = {0.0, 0.0, 0.0, 0.0};
 
 	if (hitori->board[iter.x][iter.y].status & CELL_PAINTED) {
 		painted = TRUE;
@@ -170,19 +189,24 @@ draw_cell (Hitori *hitori, GtkStyleContext *style_context, cairo_t *cr, gfloat c
 		g_debug ("State: %u", state);
 	}
 
-	/* Adwaita’s background colour. */
 	if (painted) {
-		cairo_set_source_rgb (cr, 0.956862745, 0.956862745, 0.956862745);
+		lookup_color (style_context, "painted-bg-color", &colour);
 	} else {
-		cairo_set_source_rgb (cr, 0.929411765, 0.929411765 ,0.929411765);
+		lookup_color (style_context, "unpainted-bg-color", &colour);
 	}
+
+	gdk_cairo_set_source_rgba (cr, &colour);
 	cairo_rectangle (cr, x_pos, y_pos, cell_size, cell_size);
 	cairo_fill (cr);
 
 	/* If the cell is tagged, draw the tag dots */
 	if (hitori->board[iter.x][iter.y].status & CELL_TAG1) {
-		/* Tango's lightest "sky blue" */
-		cairo_set_source_rgba (cr, 0.447058824, 0.623529412, 0.811764706, (painted == TRUE) ? PAINTED_ALPHA : NORMAL_ALPHA);
+		if (painted) {
+			lookup_color (style_context, "tag1-painted-color", &colour);
+		} else {
+			lookup_color (style_context, "tag1-unpainted-color", &colour);
+		}
+		gdk_cairo_set_source_rgba (cr, &colour);
 
 		cairo_move_to (cr, x_pos, y_pos + TAG_OFFSET);
 		cairo_line_to (cr, x_pos, y_pos);
@@ -192,8 +216,12 @@ draw_cell (Hitori *hitori, GtkStyleContext *style_context, cairo_t *cr, gfloat c
 	}
 
 	if (hitori->board[iter.x][iter.y].status & CELL_TAG2) {
-		/* Tango's lightest "chameleon" */
-		cairo_set_source_rgba (cr, 0.541176471, 0.88627451, 0.203921569, (painted == TRUE) ? PAINTED_ALPHA : NORMAL_ALPHA);
+		if (painted) {
+			lookup_color (style_context, "tag2-painted-color", &colour);
+		} else {
+			lookup_color (style_context, "tag2-unpainted-color", &colour);
+		}
+		gdk_cairo_set_source_rgba (cr, &colour);
 
 		cairo_move_to (cr, x_pos + cell_size - TAG_OFFSET, y_pos);
 		cairo_line_to (cr, x_pos + cell_size, y_pos);
@@ -204,10 +232,11 @@ draw_cell (Hitori *hitori, GtkStyleContext *style_context, cairo_t *cr, gfloat c
 
 	/* Draw the border */
 	if (painted) {
-		cairo_set_source_rgb (cr, 0.729411765, 0.737254902, 0.721568627);
+		lookup_color (style_context, "painted-border-color", &colour);
 	} else {
-		cairo_set_source_rgb (cr, 0.180392157, 0.203921569, 0.211764706);
+		lookup_color (style_context, "unpainted-border-color", &colour);
 	}
+	gdk_cairo_set_source_rgba (cr, &colour);
 	cairo_set_line_width (cr, border.left);
 	cairo_rectangle (cr, x_pos, y_pos, cell_size, cell_size);
 	cairo_stroke (cr);
@@ -221,14 +250,17 @@ draw_cell (Hitori *hitori, GtkStyleContext *style_context, cairo_t *cr, gfloat c
 	font_desc = (painted == TRUE) ? hitori->painted_font_desc : hitori->normal_font_desc;
 
 	if (hitori->board[iter.x][iter.y].status & CELL_ERROR) {
-		cairo_set_source_rgb (cr, 0.937254902, 0.160784314, 0.160784314); /* Tango's lightest "scarlet red" */
+		lookup_color (style_context, "mistaken-number-color", &colour);
+		gdk_cairo_set_source_rgba (cr, &colour);
 		pango_font_description_set_weight (font_desc, PANGO_WEIGHT_BOLD);
+	} else if (painted) {
+		lookup_color (style_context, "painted-number-color", &colour);
+		gdk_cairo_set_source_rgba (cr, &colour);
+		pango_font_description_set_weight (font_desc, PANGO_WEIGHT_NORMAL);
 	} else {
-		if (painted) {
-			cairo_set_source_rgb (cr, 0.654901961, 0.670588235, 0.654901961);
-		} else {
-			cairo_set_source_rgb (cr, 0.180392157, 0.203921569, 0.211764706);
-		}
+		g_assert (!painted);
+		lookup_color (style_context, "unpainted-number-color", &colour);
+		gdk_cairo_set_source_rgba (cr, &colour);
 		pango_font_description_set_weight (font_desc, PANGO_WEIGHT_NORMAL);
 	}
 
@@ -299,7 +331,9 @@ hitori_draw_cb (GtkWidget *drawing_area, cairo_t *cr, Hitori *hitori)
 	/* Draw a hint if applicable */
 	if (hitori->hint_status % 2 == 1) {
 		gfloat line_width = BORDER_LEFT * 2.5;
-		cairo_set_source_rgb (cr, 1, 0, 0); /* red */
+		GdkRGBA colour;
+		lookup_color (style_context, "hint-border-color", &colour);
+		gdk_cairo_set_source_rgba (cr, &colour);
 		cairo_set_line_width (cr, line_width);
 		cairo_rectangle (cr,
 				 hitori->hint_position.x * cell_size + line_width / 2,
@@ -638,7 +672,6 @@ static void
 about_cb (GSimpleAction *action, GVariant *parameters, gpointer user_data)
 {
 	HitoriApplication *self = HITORI_APPLICATION (user_data);
-	gchar *license;
 
 	const gchar *authors[] =
 	{
@@ -646,57 +679,31 @@ about_cb (GSimpleAction *action, GVariant *parameters, gpointer user_data)
 		"Ben Windsor <benjw_823@hotmail.com>",
 		NULL
 	};
-	const gchar *license_parts[] = {
-		N_("Hitori is free software: you can redistribute it and/or modify "
-		   "it under the terms of the GNU General Public License as published by "
-		   "the Free Software Foundation, either version 3 of the License, or "
-		   "(at your option) any later version."),
-		N_("Hitori is distributed in the hope that it will be useful, "
-		   "but WITHOUT ANY WARRANTY; without even the implied warranty of "
-		   "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
-		   "GNU General Public License for more details."),
-		N_("You should have received a copy of the GNU General Public License "
-		   "along with Hitori.  If not, see <http://www.gnu.org/licenses/>.")
-	};
-
-	license = g_strjoin ("\n\n",
-			  _(license_parts[0]),
-			  _(license_parts[1]),
-			  _(license_parts[2]),
-			  NULL);
 
 	gtk_show_about_dialog (GTK_WINDOW (self->window),
 				"version", VERSION,
 				"copyright", _("Copyright \xc2\xa9 2007\342\200\2232010 Philip Withnall"),
-				"comments", _("A logic puzzle designed by Nikoli."),
+				"comments", _("A logic puzzle originally designed by Nikoli"),
 				"authors", authors,
 				"translator-credits", _("translator-credits"),
 				"logo-icon-name", "hitori",
-				"license", license,
+				"license-type", GTK_LICENSE_GPL_3_0,
 				"wrap-license", TRUE,
 				"website-label", _("Hitori Website"),
 				"website", PACKAGE_URL,
 				NULL);
-
-	g_free (license);
 }
 
 static void
-board_size_activate_cb (GSimpleAction *action, GVariant *parameter, gpointer user_data)
-{
-	g_action_change_state (G_ACTION (action), parameter);
-}
-
-static void
-board_size_change_cb (GSimpleAction *action, GVariant *state, gpointer user_data)
+board_size_change_cb (GObject *object, GParamSpec *pspec, gpointer user_data)
 {
 	HitoriApplication *self = HITORI_APPLICATION (user_data);
-	const gchar *size_str;
+	gchar *size_str;
 	guint64 size;
 
-	size_str = g_variant_get_string (state, NULL);
+	size_str = g_settings_get_string (self->settings, "board-size");
 	size = g_ascii_strtoull (size_str, NULL, 10);
 	hitori_set_board_size (self, size);
 
-	g_simple_action_set_state (action, state);
+	g_free (size_str);
 }
